@@ -70,7 +70,9 @@ def linear_probe_multi_channels(hook, value_label, y, ch_inds, **regression_kwar
     value = hook.get_value_by_label(value_label)
     return linear_probe(value[:,ch_inds,:,:].values, y, **regression_kwargs)
 
-def linear_probe_single_channels(hook, value_labels, y, **regression_kwargs):
+def linear_probe_single_channels(hook, value_labels, y, 
+        value_preproc_func=lambda value: value,
+        **regression_kwargs):
     '''Probe with all individual channels in all specified values taken from the
     provided hook object.  Wrapper around linear_probe.'''
     results = []
@@ -80,7 +82,7 @@ def linear_probe_single_channels(hook, value_labels, y, **regression_kwargs):
 
         for ch_ind in tqdm(range(value.shape[1])):
             ch = value[:,ch_ind,:,:]
-            results_this = linear_probe(ch.values, y, **regression_kwargs)
+            results_this = linear_probe(value_preproc_func(ch).values, y, **regression_kwargs)
             results_this.update({
                 'value_label': value_label,
                 'channel': ch_ind,})
